#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""data.py fetches information from the data.csv and formats it to unicode and lists/dictionarys to be easily acessed by other modules."""

import operator
import csv
import os
 
def init():
    """init() reads data.csv and return a unicode list with a dictionary for every project.

    Example:
       init() returns:
       [{'project_no':1,'project_name':'Portfolio','techniques_used':['Apache','Python'],...}]"""
    file = open(os.path.dirname(__file__) + os.sep + 'data.csv', 'r')
    reader = csv.DictReader(file)
    returnlist = []
    file.seek(0)
    rows = len(file.readlines()) - 1
    file.seek(0)
    for a in xrange(rows):
        dict = reader.next()
        for element in dict:
            dict[element]=unicode(dict[element], "utf-8")
        returnlist.append(dict)
    return returnlist
    
def lookup_project(id):
    """lookup_project(...) fetches all info about the project with the specified id and returns a dictionary.

    Example:
       lookup_project(existing id) returns:
       {'project_no':1,'project_name':'Portfolio','techniques_used':['Apache','Python'],...}

       lookup_project(non-existing id) returns:
       None"""
       
    datalist = init()
    for elements in datalist:
        if(elements['project_no'] == id):
            return elements
    return None

def project_count():
    """project_count() returns the amount of projects.

    Example:
       project_count() returns:
       5 (int)"""
    datalist = init()
    count = len(datalist)
    return count
    

def retrieve_projects(sort_by='start_date', sort_order='asc', techniques=[], search=None, search_fields=None):
    """retrieve_projects(...) searches through all the projects and returns a list of all projects which matches the search parameters

    Example:
       retrieve_projects(search='existing searchword') returns:
       [{'project_no':1,'project_name':'Portfolio','techniques_used':['Apache','Python'],...}]

       retrieve_projects(search='non-existing searchword') returns:
       [] (empty list)"""
    
    datalist = init()
    retlist = []
    if search_fields == None:  #if there is no search_fields set, use these to search for:
        search_fields = ['start_date', 'end_date', 'project_no', 'project_name', 'course_id', 'course_name', 'short_description', 'long_description']
    if techniques == []:  #if there is no techniques set, use these to search for:
        techniques = retrieve_techniques()
    if search !=None:
        for project in datalist: #loop datalist to get all the projects
            for fields in search_fields: #search in these search fields
                for used_techniques in project['techniques_used'].split(","): #search for these techniques in the new list
                    if used_techniques in techniques:
                        for search_word in search.split(" "): #split every word in the search
                            for word in project[fields].split(" "): #split every word in the search field
                                if search_word.lower() in word.lower(): #check if any word in the search matches any word in the search field
                                    if retlist != []: #check to see if the return list is empty
                                        for test_project in retlist: #test to see if a project has been added into the return list allready
                                            if test_project != project: #if it hasn't then add it:
                                                retlist.append(project)
                                    else: #if the return list is empty add the project
                                        retlist.append(project)
    else: #if search is None, add all the projects to return list
        retlist = datalist
    retlist.sort(key=operator.itemgetter(sort_by)) #sort list according to sort_by
    if sort_order.lower() == "desc": #reverse the list if sort_order is desc
        retlist.reverse()
    return retlist     



def retrieve_techniques():
    """retrieve_techniques() fetches all specific techniques used

    Example:
       retrieve_techniques() returns:
       ['Apache','Python','C++']"""
    
    datalist = init()
    techniquelist = []
    for project in datalist:
        techniques = project['techniques_used'].split(",")
        for technique in techniques:
            if not technique in techniquelist:
                techniquelist.append(technique)
    techniquelist.sort()
    return techniquelist

def retrieve_technique_stats():
    """retrieve_technique_stats() uses the list from retrieve_techniques to return a list with every technique used, amount of times used and all related projects

    Example:
       retrieve_technique_stats() returns:
       [{'name':'Apache','count':3,'projects':[{'project_no':1,'project_name':'Project1'}]"""
    
    techniquelist = retrieve_techniques()
    techniquelist.sort
    datalist = init()
    returnlist = []
    for technique in techniquelist:
        projlist = []
        for project in datalist:
            if technique in project['techniques_used']:
                projlist.append({'project_no':project['project_no'],'project_name':project['project_name']})
        projcount = len(projlist)
        returnlist.append({'name':technique,'count':projcount,'projects':projlist})
    return returnlist
